----------------------------
--      Constants         --
----------------------------

local CASTING_SOUND_FILE = "Interface\\AddOns\\FocusInterruptSounds\\casting.wav"
local INTERRUPTED_SOUND_FILE = "Interface\\AddOns\\FocusInterruptSounds\\interrupted.wav"

local SCHOOL_PHYSICAL	= 0x01;
local SCHOOL_HOLY	= 0x02;
local SCHOOL_FIRE	= 0x04;
local SCHOOL_NATURE	= 0x08;
local SCHOOL_FROST	= 0x10;
local SCHOOL_SHADOW	= 0x20;
local SCHOOL_ARCANE	= 0x40;
local SCHOOL_ALL	= 0x7F;

------------------------------
--      Initialization      --
------------------------------

FocusInterruptSounds = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0")

function FocusInterruptSounds:OnInitialize()

end


function FocusInterruptSounds:OnEnable()
	_, self.strClassName = UnitClass("player");

	if ("WARLOCK" == self.strClassName) then
		self.strInterruptSpellName = "Spell Lock";
		self.iInterruptSchool = SCHOOL_SHADOW;
		self.fUsePetInterrupt = true;
	elseif ("MAGE" == self.strClassName) then
		self.strInterruptSpellName = "Counterspell";
		self.iInterruptSchool = SCHOOL_ARCANE;
		self.fUsePetInterrupt = false;
	elseif ("SHAMAN" == self.strClassName) then
		self.strInterruptSpellName = "Earth Shock";
		self.iInterruptSchool = SCHOOL_NATURE;
		self.fUsePetInterrupt = false;
	elseif ("WARRIOR" == self.strClassName) then
		self.strInterruptSpellName = "Pummel";
		self.iInterruptSchool = SCHOOL_PHYSICAL;
		self.fUsePetInterrupt = false;
	elseif ("ROGUE" == self.strClassName) then
		self.strInterruptSpellName = "Kick";
		self.iInterruptSchool = SCHOOL_PHYSICAL;
		self.fUsePetInterrupt = false;
	elseif ("PRIEST" == self.strClassName) then
		self.strInterruptSpellName = "Silence";
		self.iInterruptSchool = SCHOOL_SHADOW;
		self.fUsePetInterrupt = false;
	elseif ("HUNTER" == self.strClassName) then
		self.strInterruptSpellName = "Silencing Shot";
		self.iInterruptSchool = SCHOOL_PHYSICAL;
		self.fUsePetInterrupt = false;
	end

	-- Only do stuff for classes with an interrupted or silence
	if (nil ~= self.strInterruptSpellName) then
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		self:PrintMessage("Add-on activated for the class " .. self.strClassName);
	else
		self:PrintMessage("Add-on disabled for the class " .. self.strClassName);
	end
end

function FocusInterruptSounds:OnDisable()

end

function FocusInterruptSounds:PrintMessage(msg)
	DEFAULT_CHAT_FRAME:AddMessage("|cff7fff7fFocusInterruptSounds|r: " .. msg);
end

function FocusInterruptSounds:FBlacklistSpell(strSpellId, strSpellName, iSpellSchool)
	if (0 == bit.band(iSpellSchool, bit.bnot(SCHOOL_PHYSICAL))
		or "Mass Dispel" == strSpellName
		or "Starfire" == strSpellName
		or "Immolate" == strSpellName
		or "Searing Pain" == strSpellName
		or "Empowered Smite" == strSpellName
		or "Divine Wrath" == strSpellName
	) then
		return true;
	end

	return false;
end

function FocusInterruptSounds:FChanneledSpell(strSpellId, strSpellName, iSpellSchool)
	if ("Drain Life" == strSpellName
		or "Drain Mana" == strSpellName
		or "Mind Flay" == strSpellName
		or "Tranquility" == strSpellName
		or "Arcane Missiles" == strSpellName
	) then
		return true;
	end

	return false;
end

function FocusInterruptSounds:FIsFocusImmune(iSpellSchool)
	for i = 1, 40 do
		local strBuffName, _, _, _, _, _ = UnitBuff("focus", i);
		if ("Divine Shield" == strBuffName
			or "Divine Protection" == strBuffName
			or (0 ~= bit.band(iSpellSchool, SCHOOL_PHYSICAL) and "Blessing of Protection" == strBuffName)
			or (0 ~= bit.band(iSpellSchool, SCHOOL_SHADOW) and "Nether Protection" == strBuffName)
		) then
			return true;
		end
	end

	return false;
end

function FocusInterruptSounds:FIsPetSpellAvailable(strSpellName)

	-- Verify that the pet can act (i.e. isn't feared)
	if (not GetPetActionsUsable()) then
		return false;
	end

	-- Verify that the spell isn't on cooldown (also checks existence)
	iStartTime, _, fSpellEnabled = GetSpellCooldown(strSpellName, BOOKTYPE_PET);
	if (iStartTime ~= 0 or not fSpellEnabled) then
		return false;
	end

	-- Verify mana/energy
	_, _, _, iCost, _, _, _, _, _ = GetSpellInfo(strSpellName);
	if (UnitMana("playerpet") < iCost) then
		return false;
	end

	return true;
end

function FocusInterruptSounds:FIsSpellAvailable(strSpellName)

	-- Verify that the spell isn't on cooldown
	iStartTime, _, fSpellEnabled = GetSpellCooldown(strSpellName);
	if (iStartTime ~= 0 or not fSpellEnabled) then
		return false;
	end

	-- Verify mana/energy
	_, _, _, iCost, _, _, _, _, _ = GetSpellInfo(strSpellName);
	if (UnitMana("player") < iCost) then
		return false;
	end

	return true;
end

function FocusInterruptSounds:FIsMobBlacklisted(strMobName)

	if ("Sunblade Cabalist" == strMobName
		or "Sunblade Dawn Priest" == strMobName
		or "Sunblade Dusk Priest" == strMobName
		or "Coilskar Sea-Caller" == strMobName
		or "Coilskar Soothsayer" == strMobName
		or "Shadowmoon Blood Mage" == strMobName
		or "Shadowmoon Deathshaper" == strMobName
		or "Bonechewer Blood Prophet" == strMobName
	) then
		return true;
	end

	return false;
end

function FocusInterruptSounds:COMBAT_LOG_EVENT_UNFILTERED(iTimestamp, strEventType, strSourceGuid, strSourceName, iSourceFlags, strDestGuid, strDestName, iDestFlags, varParam1, varParam2, varParam3, varParam4, varParam5, varParam6, ...)

	-- Play a sound when the Focus starts casting
	if (0 ~= bit.band(iSourceFlags, COMBATLOG_OBJECT_REACTION_HOSTILE)
		and 0 ~= bit.band(iSourceFlags, COMBATLOG_OBJECT_FOCUS)
		and (("SPELL_CAST_START" == strEventType and not self:FBlacklistSpell(varParam1, varParam2, varParam3))
			or ("SPELL_CAST_SUCCESS" == strEventType and self:FChanneledSpell(varParam1, varParam2, varParam3)))
		and not self:FIsFocusImmune(self.iInterruptSchool)
		and (0 == bit.band(iSourceFlags, COMBATLOG_OBJECT_CONTROL_NPC) or not self:FIsMobBlacklisted(strSourceName))
	) then
		if ((self.fUsePetInterrupt and self:FIsPetSpellAvailable(self.strInterruptSpellName))
			or (not self.fUsePetInterrupt and self:FIsSpellAvailable(self.strInterruptSpellName))
		) then
			self:PrintMessage(strSourceName .. " is casting |cffff4444" .. varParam2 .. "|r!");
			PlaySoundFile(CASTING_SOUND_FILE);
		end
	end

	-- Play sound when the Focus is interrupted
	if ("SPELL_INTERRUPT" == strEventType
		and 0 ~= bit.band(iSourceFlags, COMBATLOG_OBJECT_AFFILIATION_MINE)
		and 0 ~= bit.band(iDestFlags, COMBATLOG_OBJECT_REACTION_HOSTILE)
		and 0 ~= bit.band(iDestFlags, COMBATLOG_OBJECT_FOCUS)
	) then
		self:PrintMessage("You've successfully interrupted |cffaaffff" .. varParam5 .. "|r.");
		PlaySoundFile(INTERRUPTED_SOUND_FILE);
	end
end
